<?php
/**
 * Better Together for PrestaShop
 * By Scott Wilson (swguy) 
 * http://www.thatsoftwareguy.com
 * Version 0.1 
 * @copyright Copyright 2009, That Software Guy
 */

  function bt_cmp($a, $b) {
       if ($a['price'] == $b['price']) return 0; 
       if ($a['price'] < $b['price']) return 1; 
       return -1; 
  }

  // Make it pretty obvious that there's a problem
  function bailout($str) { 
      trigger_error($str); 
      die($str); 
  }

  define('PROD_TO_PROD', '1');
  define('PROD_TO_CAT', '2');
  define('CAT_TO_CAT', '3');
  define('CAT_TO_PROD', '4');

  define('TWOFER_PROD', '11');
  define('TWOFER_CAT', '12');

  class bt_discount {
     function init($ident1, $ident2, $type, $amt, $flavor) {
          $this->isvalid = 0; 
          if ($type != "$" && $type != "%") {
              bailout("Bad type " . $type); 
          }
          if ($flavor != PROD_TO_PROD && 
              $flavor != PROD_TO_CAT &&
              $flavor != CAT_TO_PROD &&
              $flavor != CAT_TO_CAT) {
              bailout("Bad flavor " . $flavor); 
          }
          $this->ident1 = $ident1; // Product id or category
          $this->ident2 = $ident2; // Product id or category 
          $this->type = $type; // % or $ 
          $this->amt = $amt;   // numerical amount 
          $this->flavor = $flavor; // PROD_TO_PROD, PROD_TO_CAT, CAT_TO_CAT, CAT_TO_PROD
          $this->isvalid = 1; 
     } 
     function getid() {
          return $this->ident1;
     } 
  }

  class bt_twofer{
     function init($ident1, $flavor) {
          $this->isvalid = 0; 
          if ($flavor != TWOFER_PROD && 
              $flavor != TWOFER_CAT) {
              bailout("Bad flavor " . $flavor); 
          }
          $this->ident1 = $ident1; // Product id or category
          $this->flavor = $flavor; // PROD, CAT
          $this->isvalid = 1; 
     } 
     function getid() {
          return $this->ident1;
     } 
  }

  class better_together {

    function better_together() {
      $this->output = array();
      $this->discountlist = array();
      $this->twoferlist = array();
      $this->nocontext = 0; 
      $this->setup(); 
    }
   
    function setnocontext() { 
      $this->nocontext = 1; 
    } 

    function add_twoforone_prod($ident1) { 
         $d = new bt_twofer; 
         $d->init($ident1, TWOFER_PROD);
         if ($d->isvalid == 1) { 
            $this->twoferlist[] = & $d; 
         }
    }

    function add_twoforone_cat($ident1) { 
         $d = new bt_twofer; 
         $d->init($ident1, TWOFER_CAT);
         if ($d->isvalid == 1) { 
            $this->twoferlist[] = & $d; 
         }
    }
 
    function add_prod_to_prod($ident1, $ident2, $type, $amt)  {
         $d = new bt_discount; 
         $d->init($ident1, $ident2, $type, $amt, PROD_TO_PROD);
         if ($d->isvalid == 1) { 
            $this->discountlist[] = & $d; 
         }
    }

    function add_prod_to_cat($ident1, $ident2, $type, $amt)  {
         $d = new bt_discount; 
         $d->init($ident1, $ident2, $type, $amt, PROD_TO_CAT);
         if ($d->isvalid == 1) { 
            $this->discountlist[] = & $d; 
         }
    }

    function add_cat_to_cat($ident1, $ident2, $type, $amt)  {
         $d = new bt_discount; 
         $d->init($ident1, $ident2, $type, $amt, CAT_TO_CAT);
         if ($d->isvalid == 1) { 
            $this->discountlist[] = & $d; 
         }
    }

    function add_cat_to_prod($ident1, $ident2, $type, $amt)  {
         $d = new bt_discount; 
         $d->init($ident1, $ident2, $type, $amt, CAT_TO_PROD);
         if ($d->isvalid == 1) { 
            $this->discountlist[] = & $d; 
         }
    }


    function print_amount($amount) {
      global $db, $order, $currencies;
      return  $currencies->format($amount, true, $order->info['currency'], $order->info['currency_value']);
    }

    function get_discount($discount_item, &$all_items, $useTax) {
        $discount = 0; 
        for ($dis=0, $n=count($this->discountlist); $dis<$n; $dis++) {
           $li = $this->discountlist[$dis]; 

           // Based on type, check ident1
           if ( ($li->flavor == PROD_TO_PROD) || 
                ($li->flavor == PROD_TO_CAT) ) {
              if ($li->ident1 != $discount_item['id_product']) {
                 continue;
              }
           } else { // CAT_TO_CAT, CAT_TO_PROD
              if ($li->ident1 != $discount_item['id_category_default']) {
                 continue;
              }
           }

           for ($i=sizeof($all_items) - 1; $i>= 0; $i--) {
               if ($all_items[$i]['cart_quantity'] == 0) 
                  continue;
               $match = 0; 
               if ( ($li->flavor == PROD_TO_PROD) || 
                    ($li->flavor == CAT_TO_PROD) ) {
                  if ($all_items[$i]['id_product'] == $li->ident2) {
                     $match = 1;
                  }
               } else { // CAT_TO_CAT, PROD_TO_CAT 
                  if ($all_items[$i]['id_category_default'] == $li->ident2) {
                     $match = 1;
                  }
               }

               if ($match == 1) { 
                   $all_items[$i]['cart_quantity'] -= 1;
                   if ($li->type == "$") {
                      $discount = $li->amt;
                   } else {
                      $discount = $all_items[$i]['price'] *  
                                  $li->amt / 100;
                   }
                   $discount = round($discount, 2); 
                   if ($useTax) {
                      $discount = $this->gross_up($discount, $all_items[$i]['rate']); 
                   }
                   return $discount;
               }
           }
        }

        return 0;
    }

    function is_twofer($discount_item) {
        $discount = 0; 
        for ($dis=0, $n=count($this->twoferlist); $dis<$n; $dis++) {
           $li = $this->twoferlist[$dis]; 

           // Based on type, check ident1
           if ( ($li->flavor == TWOFER_PROD) && 
                ($li->ident1 == $discount_item['id_product']) ) {
                 return true;
           } else if ( ($li->flavor == TWOFER_CAT) && 
                       ($li->ident1 == $discount_item['id_category_default']) ) {
                 return true;
           }
       }
       return false;
    }

    function calculate_deductions($products, $useTax = true) {
       $rc = usort($products, "bt_cmp");
       $discountable_products = array();
       // Build discount list 
       for ($i=0, $n=sizeof($products); $i<$n; $i++) {
            $discountable_products[$i] = $products[$i]; 
       }

       // Now compute discounts
       $discount = 0;
       for ($i=0, $n=sizeof($discountable_products); $i<$n; $i++) {
            // Is it a twofer? 
            if ($this->is_twofer($discountable_products[$i])) {
                $npairs = (int)($discountable_products[$i]['cart_quantity']/2);
                $discountable_products[$i]['cart_quantity'] -= ($npairs * 2);
                $item_discountable = $npairs * $discountable_products[$i]['price'];
                $item_discountable = round($item_discountable, 2);
                if ($useTax) {
                      $discount += $this->gross_up($item_discountable, $discountable_products['rate']); 
                } else {
                      $discount += $item_discountable;
                }
            }

            // Otherwise, do regular bt processing
            while ($discountable_products[$i]['cart_quantity'] > 0) { 
                $discountable_products[$i]['cart_quantity'] -= 1;
                $item_discountable = $this->get_discount(       
                    $discountable_products[$i], $discountable_products, $useTax);
                if ($item_discountable == 0) { 
                    $discountable_products[$i]['cart_quantity'] += 1;
                    break;
                } else {
                      $discount += $item_discountable;
                }
            }
       }
       return $discount; 
    }

    function gross_up($net, $rate) {
       if ($rate == 0) { 
            return $net;
       }
       $gross_up_amt += round((($net * $rate)) /100, 2) ;
       return $gross_up_amt + $net; 
    }

    function setup() {
         // Add all linkages here
         // Some examples are provided: 
         // Buy product 7, get product 2 at 50% off 
         // $this->add_prod_to_prod(7, 2, "%", 100); 
         // $this->add_cat_to_cat(2, 2, "%", 50); 
         // $this->add_twoforone_cat(2);
         // $this->add_twoforone_prod(2);
         $this->add_prod_to_prod(2, 2, "%", 100); 
    }

  }
?>
